<?php
define('OP5LICENSE_CACHE_FILE', './op5license-cache');
define('OP5LICENSE_FILE', '/etc/op5license/op5license.xml');
define('OP5LICENSE_ERR_LICENSE_OK', '1');
define('OP5LICENSE_ERR_EXCEEDS_LICENSE', '0');
define('OP5LICENSE_ERR_UNLICENSED_PRODUCT', '2');
define('OP5LICENSE_TYPE_REGULAR', '1');
define('OP5LICENSE_TYPE_SITE', '2');
define('OP5LICENSE_TYPE_EVALUATION', '3');
define('OP5LICENSE_TYPE_UNLICENSED', '4');
define('OP5LICENSE_TYPE_ENTERPRISE', '5');
define('OP5LICENSE_TYPE_BASIC', '6');
define('OP5LICENSE_TYPE_FREE', '7');
define('OP5LICENSE_CHECK_OK', '1');
define('OP5LICENSE_CHECK_ERROR', '0');
define('OP5LICENSE_CHECK_EVALUATION', '2');

class op5license
{
	private $license_file;
	private $license_data;
	public  $license_status = NULL;
	private $license;
	private $verify_output;
	private $verify_return;
	private $products = array();
	private $usage = array();
	private $cache = array();
	private $errors = array();
	private $evaluations = array();
	private $monitor_hosts = 20;
	private $logserver_hosts = 10;
	private $statistics_hosts = 10;

	public function __construct($override_file = FALSE)
	{
		$file = ($override_file) ? $override_file : OP5LICENSE_FILE;

		if (!$this->set_license_file($file))
			$this->error('License file does not exist. A license file needs to be installed.');

		if (!$this->load_license_from_file())
			$this->error('Invalid license format. The file is not a valid op5 license file, or the file has been tampered with.');

		if (!$this->verify_signature())
			$this->error('Signature does not match license data. The license file is in a valid format, but the signature or the license data has been tampered with.');

		$this->parse_license();
	}

	private function op5license_Monitor()
	{
		if (!is_file("/opt/monitor/var/objects.cache"))
			return -1;
		else
		{
			exec('grep -c "define host " /opt/monitor/var/objects.cache', $count_output, $count_return);
			return ($count_return == 0) ? $count_output[0] : -2;
		}
	}

	private function op5license_Logserver()
	{
		$logserver_cache_file = '/tmp/logserver-license.dat';

		if (!file_exists($logserver_cache_file))
			return -1;

		$fp = fopen($logserver_cache_file, 'r');
		$in = fread($fp, filesize($logserver_cache_file));
		$in = rtrim($in);

		$ary = explode(':', $in);

		$num = count($ary);

		return $num;
	}

	private function op5license_Statistics()
	{
		//require_once("/opt/statistics/include/config.php");
		$database_default = 'statistics';
		$database_hostname = 'localhost';
		$database_username = 'stats';
		$database_password = 'stats';

		if (!$op5l_db = @mysql_connect($database_hostname, $database_username, $database_password))
			return -1;

		if (!mysql_select_db($database_default, $op5l_db))
			return -2;

		if (!$res = mysql_query('SELECT count(id) FROM host', $op5l_db))
			return -3;

		$count = mysql_fetch_assoc($res);

		return isset($count['count(id)']) ? $count["count(id)"] : 0;
	}

	private function error($message, $product = NULL)
	{
		$this->errors[isset($product) ? $product : 'General'][] = $message;
	}

	private function set_license_file($file)
	{
		return !is_null($this->license_file = file_exists($file) ? $file : NULL);
	}

	public function get_license_file()
	{
		return ($this->license != NULL) ? $this->license : FALSE;
	}

	public function license_file_exists()
	{
		return is_file($this->license_file);
	}

	private function verify_signature()
	{
		exec("/opt/op5sys/bin/op5license-verify {$this->license_file}", $verify_output, $verify_return);
		return (bool)($verify_return == 0);
	}

	private function load_license_from_file()
	{
		if ($this->license_file) {
			$license = @simplexml_load_file($this->license_file);
			if ((bool)($license)) {
				$this->license = $license;
				return true;
			}
		}
		return false;
	}

	private function parse_license()
	{
		if (isset($this->license))
		{
			foreach ($this->license->item as $item)
				$this->license_data[(string)$item['name']] = (string)$item['value'];

			return TRUE;
		}
		else
			return FALSE;
	}

	public function register_product($product_name, $override_usage_callback = NULL)
	{
		if (isset($override_usage_callback))
			$this->products[$product_name] = $override_usage_callback;
		else
		{
			$callback = 'op5license_' . $product_name;
			$this->products[$product_name] = $callback;
			require_once($callback . '.php');
		}
	}

	private function check_monitor_product_type() 
	{
		/* If MonitorProductType is set in the license we use it to decide the type */
		if (isset( $this->license_data['MonitorProductType'])) {
			switch ($this->license_data['MonitorProductType']) {
			 case 'Enterprise': 
				return 'Enterprise';
			 case 'Basic':
				return 'Basic';
			 default:
				return 'Free';
			}
		}
		/* If MonitorProductType is not set we us MonitorLicenseType. Regular & Site is considered Enterprise */
		switch ($this->license_data['MonitorLicenseType'])
		{	
		 case 'Regular':
		 case 'Site':
		 case 'Enterprise':
			return 'Enterprise';
		 case 'Basic':
			return 'Basic';
		}
		return 'Free';
	}

	private function check_product_license_type($product)
	{
		switch ($this->license_data[$product . 'LicenseType'])
		{
			case 'Regular':
				return OP5LICENSE_TYPE_REGULAR;

			case 'Site':
				return OP5LICENSE_TYPE_SITE;

			case 'Evaluation':
				return OP5LICENSE_TYPE_EVALUATION;

			case 'Enterprise':
				return OP5LICENSE_TYPE_ENTERPRISE;

			case 'Basic':
				return OP5LICENSE_TYPE_BASIC;

			case 'Free':
				return OP5LICENSE_TYPE_FREE;

			default:
				return OP5LICENSE_TYPE_UNLICENSED;
		}
	}

	private function check_product_usage($product, $callback)
	{
		if (!isset($this->usage[$product]))
			$this->usage[$product] = call_user_func($callback);

		if ($this->check_product_license_type($product) == OP5LICENSE_TYPE_REGULAR || 
			$this->check_product_license_type($product) == OP5LICENSE_TYPE_ENTERPRISE ||
			$this->check_product_license_type($product) == OP5LICENSE_TYPE_BASIC)
		{
			$license_data_string = $product . 'Hosts';

			if (!isset($this->license_data[$license_data_string]))
				return OP5LICENSE_ERR_UNLICENSED_PRODUCT;
			elseif($this->usage[$product] <= $this->license_data[$license_data_string])
				return OP5LICENSE_ERR_LICENSE_OK;
			else
				return OP5LICENSE_ERR_EXCEEDS_LICENSE;
		}
		elseif ($this->check_product_license_type($product) == OP5LICENSE_TYPE_UNLICENSED)
			return OP5LICENSE_ERR_UNLICENSED_PRODUCT;
		else
			return OP5LICENSE_ERR_LICENSE_OK;
	}

	public function get_customer_info()
	{
		$customer['CustomerName'] = $this->license_data['IssuedToName'];
		$customer['CustomerEmail'] = $this->license_data['IssuedToEmail'];
		$customer['LicenseIssueDate'] = $this->license_data['IssueDate'];
		if (substr($this->license_data['IssuedToCustomerNumber'], -3, 1) == '-')
		{
			$customer['CustomerNumber'] = substr($this->license_data['IssuedToCustomerNumber'], 0, -3);
			$customer['LicenseKeyNumber'] = substr($this->license_data['IssuedToCustomerNumber'], -2);
		}
		else
			$customer['CustomerNumber'] = $this->license_data['IssuedToCustomerNumber'];

		return $customer;
	}

	public function get_errors()
	{
		return $this->errors;
	}

	public function get_products()
	{
		if (count($this->usage) < count($this->products))
			$this->check();

		$p = array();
		foreach ($this->usage as $product => $current_usage)
		{
			$p[$product]['name'] = $product;
			$p[$product]['license_type'] = $this->check_product_license_type($product);
			$p[$product]['product_type'] = $this->check_monitor_product_type();
			$p[$product]['current_usage'] = $current_usage;

			switch ($p[$product]['license_type'])
			{
				case OP5LICENSE_TYPE_REGULAR:
					$p[$product]['license_type_str'] = 'Regular';
					$p[$product]['max_usage'] = $this->license_data[$product . 'Hosts'];
					break;
				case OP5LICENSE_TYPE_ENTERPRISE:
					$p[$product]['license_type_str'] = 'Enterprise';
					$p[$product]['max_usage'] = $this->license_data[$product . 'Hosts'];
					break;
				case OP5LICENSE_TYPE_BASIC:
					$p[$product]['license_type_str'] = 'Basic';
					$p[$product]['max_usage'] = $this->license_data[$product . 'Hosts'];
					break;

				case OP5LICENSE_TYPE_SITE:
					$p[$product]['license_type_str'] = 'Site';
					break;

				case OP5LICENSE_TYPE_EVALUATION:
					$p[$product]['license_type_str'] = 'Evaluation';
					break;
				case OP5LICENSE_TYPE_FREE:
					$p[$product]['license_type_str'] = 'Free';
					break;

				default:
					$p[$product]['license_type_str'] = 'Unlicensed';
			}

			if (!empty($this->errors[$product]))
				$p[$product]['errors'] = $this->errors[$product];
		}

		return $p;
	}

	public function check($check_usage = TRUE)
	{
		if (count($this->products) > 0)
			foreach ($this->products as $product => $callback)
			{
				$type = $this->check_product_license_type($product);

				switch ($product)
				{
					case 'Monitor' :
						$number_of_hosts = $this->op5license_Monitor();
						$max_hosts = $this->monitor_hosts;
						break;

					case 'Logserver' :
						$number_of_hosts = $this->op5license_Logserver();
						$max_hosts = $this->logserver_hosts;
						break;

					case 'Statistics' :
						$number_of_hosts = $this->op5license_Statistics();
						$max_hosts = $this->statistics_hosts;
				}

				if ($check_usage != FALSE)
					$usage = $this->check_product_usage($product, $callback);
				else
				{
					$usage = OP5LICENSE_ERR_LICENSE_OK;
					$this->usage[$product] = 0;
				}

				if ($usage == OP5LICENSE_ERR_EXCEEDS_LICENSE)
					$this->error('Usage exceeds license!', $product);
				elseif ($usage == OP5LICENSE_ERR_UNLICENSED_PRODUCT)
					$this->error('This product is unlicensed.', $product);

				if ($type == OP5LICENSE_TYPE_EVALUATION)
					$this->evaluations[$product] = 1;
			}
		else
		{
			$number_of_hosts = 0;
			$max_hosts = 0;
		}

		if (count($this->errors) && $this->license_file)
			return $this->license_status = OP5LICENSE_CHECK_ERROR;
		elseif (count($this->evaluations) && $number_of_hosts > $max_hosts)
			return $this->license_status = OP5LICENSE_CHECK_EVALUATION;
		elseif (!$this->license_file && $number_of_hosts > $max_hosts)
			return $this->license_status = OP5LICENSE_CHECK_ERROR;
		else
			return $this->license_status = OP5LICENSE_CHECK_OK;
	}

	public function check_Monitor_allowed()
	{
		if ($this->check_product_license_type('Monitor') == OP5LICENSE_TYPE_SITE){
			return 0;
		}
		if(isset($this->license_data['MonitorHosts']))
			return $this->license_data['MonitorHosts'];
		return 0;
	}

	public function status()
	{
		if ($this->license_status == NULL)
		  $this->check();

		return $this->license_status;
	}

	private function license_bar()
	{
		if ($this->license_status == OP5LICENSE_CHECK_OK)
			return NULL;

		$license_info = '<div id="licenseInfo" onclick="javascript:top.location.href=\'/license.php\'" style="background-color: #333333; height: 24px; padding: 18px 0px; font-size: 16px">';
		if ($this->license_status == OP5LICENSE_CHECK_ERROR)
			$license_info .= '<strong>You have an invalid license.</strong>';
		elseif ($this->license_status == OP5LICENSE_CHECK_EVALUATION)
			$license_info .= '<strong>You have an evaluation license.</strong>';
		$license_info .= '<span style="font-family: arial; padding-left: 10px; color: #ffffff">To get access to updates via op5 and the op5 support services, you need to purchase a valid license. Click here for more information.</span></div>';

		return $license_info;
	}

	public function print_license_bar()
	{
		echo $this->license_bar();
	}
}
?>
